@@ -4,11 +4,12 @@ import logging
 import typing
 
 from Crypto.Cipher import AES
-from Crypto.Util.Padding import pad, unpad
+from Crypto.Util.Padding import unpad
 
 from roborock.cloud_api import RoborockMqttClient
 from roborock.containers import DeviceData, RoborockCategory, UserData
 from roborock.exceptions import RoborockException
+from roborock.protocols.a01_protocol import encode_mqtt_payload
 from roborock.roborock_message import (
     RoborockDyadDataProtocol,
     RoborockMessage,
@@ -38,12 +39,11 @@ class RoborockMqttClientA01(RoborockMqttClient, RoborockClientA01):
         self.queue_timeout = queue_timeout
         self._logger = RoborockLoggerAdapter(device_info.device.name, _LOGGER)
 
-    async def send_message(self, roborock_message: RoborockMessage):
+    async def _send_message(self, roborock_message: RoborockMessage):
         await self.validate_connection()
         response_protocol = RoborockMessageProtocol.RPC_RESPONSE
 
         m = self._encoder(roborock_message)
-        # self._logger.debug(f"id={request_id} Requesting method {method} with {params}")
         payload = json.loads(unpad(roborock_message.payload, AES.block_size))
         futures = []
         if "10000" in payload["dps"]:
@@ -56,7 +56,6 @@ class RoborockMqttClientA01(RoborockMqttClient, RoborockClientA01):
             for i, dps in enumerate(json.loads(payload["dps"]["10000"])):
                 response = responses[i]
                 if isinstance(response, BaseException):
-                    self._logger.warning("Timed out get req for %s after %s s", dps, self.queue_timeout)
                     dps_responses[dps] = None
                 else:
                     dps_responses[dps] = response
@@ -65,24 +64,14 @@ class RoborockMqttClientA01(RoborockMqttClient, RoborockClientA01):
     async def update_values(
         self, dyad_data_protocols: list[RoborockDyadDataProtocol | RoborockZeoProtocol]
     ) -> dict[RoborockDyadDataProtocol | RoborockZeoProtocol, typing.Any]:
-        payload = {"dps": {RoborockDyadDataProtocol.ID_QUERY: str([int(protocol) for protocol in dyad_data_protocols])}}
-        return await self.send_message(
-            RoborockMessage(
-                protocol=RoborockMessageProtocol.RPC_REQUEST,
-                version=b"A01",
-                payload=pad(json.dumps(payload).encode("utf-8"), AES.block_size),
-            )
+        message = encode_mqtt_payload(
+            {RoborockDyadDataProtocol.ID_QUERY: str([int(protocol) for protocol in dyad_data_protocols])}
         )
+        return await self._send_message(message)
 
     async def set_value(
         self, protocol: RoborockDyadDataProtocol | RoborockZeoProtocol, value: typing.Any
     ) -> dict[int, typing.Any]:
         """Set a value for a specific protocol on the A01 device."""
-        payload = {"dps": {int(protocol): value}}
-        return await self.send_message(
-            RoborockMessage(
-                protocol=RoborockMessageProtocol.RPC_REQUEST,
-                version=b"A01",
-                payload=pad(json.dumps(payload).encode("utf-8"), AES.block_size),
-            )
-        )
+        message = encode_mqtt_payload({protocol: value})
+        return await self._send_message(message)
