@@ -6,7 +6,7 @@ from unittest.mock import AsyncMock, Mock, patch
 import pytest
 
 from roborock.containers import HomeData, UserData
-from roborock.devices.device import DeviceVersion
+from roborock.devices.cache import CacheData, InMemoryCache
 from roborock.devices.device_manager import create_device_manager, create_home_data_api
 from roborock.exceptions import RoborockException
 
@@ -63,13 +63,11 @@ async def test_with_device() -> None:
     assert len(devices) == 1
     assert devices[0].duid == "abc123"
     assert devices[0].name == "Roborock S7 MaxV"
-    assert devices[0].device_version == DeviceVersion.V1
 
     device = await device_manager.get_device("abc123")
     assert device is not None
     assert device.duid == "abc123"
     assert device.name == "Roborock S7 MaxV"
-    assert device.device_version == DeviceVersion.V1
 
     await device_manager.close()
 
@@ -95,9 +93,43 @@ async def test_home_data_api_exception() -> None:
 async def test_create_home_data_api_exception() -> None:
     """Test that exceptions from the home data API are propagated through the wrapper."""
 
-    with patch("roborock.devices.device_manager.RoborockApiClient.get_home_data") as mock_get_home_data:
+    with patch("roborock.devices.device_manager.RoborockApiClient.get_home_data_v3") as mock_get_home_data:
         mock_get_home_data.side_effect = RoborockException("Test exception")
         api = create_home_data_api(USER_DATA, mock_get_home_data)
 
         with pytest.raises(RoborockException, match="Test exception"):
             await api()
+
+
+async def test_cache_logic() -> None:
+    """Test that the cache logic works correctly."""
+    call_count = 0
+
+    async def mock_home_data_with_counter() -> HomeData:
+        nonlocal call_count
+        call_count += 1
+        return HomeData.from_dict(mock_data.HOME_DATA_RAW)
+
+    class TestCache:
+        def __init__(self):
+            self._data = CacheData()
+
+        async def get(self) -> CacheData:
+            return self._data
+
+        async def set(self, value: CacheData) -> None:
+            self._data = value
+
+    # First call happens during create_device_manager initialization
+    device_manager = await create_device_manager(USER_DATA, mock_home_data_with_counter, cache=InMemoryCache())
+    assert call_count == 1
+
+    # Second call should use cache, not increment call_count
+    devices2 = await device_manager.discover_devices()
+    assert call_count == 1  # Should still be 1, not 2
+    assert len(devices2) == 1
+
+    await device_manager.close()
+    assert len(devices2) == 1
+
+    await device_manager.close()
